#!/bin/bash

. /etc/default/${EPAGES_PROJECT:-epages6}
. /var/epages/functions/fun-pkg.sh
export pkg_installed required_pkg_version

PERL_EXE=$EPAGES_PERL/bin/perl

ERROR=$(ls Makefile.PL patch.pl $PERL_EXE 2>&1 >/dev/null)
RESULT=$?
if [[ $RESULT != 0 ]] ; then
  echo "patch.sh: $ERROR"
  exit $RESULT
fi

hostname=$(hostname)
PATCH_TAG=$(awk -F= '/^PATCH_TAG[[:space:]]*=/{gsub(/[[:space:]]+/,""); print $2}' Makefile.PL)
LOG_DIR=${EPAGES_PATCHES:-"$EPAGES_SHARED/Patches"}/var-$PATCH_TAG/logs/$hostname
mkdir -m 755 -p $LOG_DIR || exit 1

#------------------------------------------------------------------------------
# test if package is installed in required version
#------------------------------------------------------------------------------

test_package() {
  local PACKAGE=$1
  local REQ_VERSION=$2

  local VER
  if [[ -f /etc/debian_version ]] && dpkg --status $PACKAGE >/dev/null 2>&1 ; then
    VER=$(dpkg-query --show -f '${Version}\n' $PACKAGE | sed 's,[~-].*,,' | sort -V | tail -1)
  elif [[ -f /etc/redhat-release ]] && rpm --quiet -q $PACKAGE ; then
    VER=$(rpm -q $PACKAGE --qf '[%{VERSION}\n]' | sort -V | tail -1)
  fi

  [[ -n $VER ]] || return 1
  [[ $(echo -e "$VER\n$REQ_VERSION" | sort -V | head -1) = $REQ_VERSION ]]
  echo $VER
  return $?
}

#------------------------------------------------------------------------------
# check if epages-release is up-to-date
#------------------------------------------------------------------------------

# test if function required_pkg_version exists
if ! type -t required_pkg_version >/dev/null ; then
  INSTALLED_VERSION=$(test_package epages-release 6.17.36)
  echo "Package epages-release (${INSTALLED_VERSION:-not found}) is outdated, exiting."
  echo "First update epages-release by executing:"
  if [[ -f /etc/debian_version && -z $INSTALLED_VERSION ]] ; then
    echo "  curl -O  http://epages.com/repo/epages/apt/epages-release_all.deb"
    echo "  dpkg -i epages-release_all.deb ; apt-get -y update"
  else
    echo "  rpm -hUv http://epages.com/repo/epages/yum/epages-release.noarch.rpm"
  fi
  echo "Then re-run: $0 $@"
  exit 1
fi

#------------------------------------------------------------------------------
# check if epages-perl is up-to-date
#------------------------------------------------------------------------------

# do not update perl if perl flex server is running
service ePagesConfigServer status >/dev/null 2>&1 && SKIP_PERL_UPDATE=exit

# if patch.sh was executed by a packaging tool we can't run packaging tools
started_by_rpm $$ && SKIP_PERL_UPDATE=rpm

if ! required_pkg_version epages-perl 6.17.36 ; then
  if [[ $SKIP_PERL_UPDATE = rpm ]] ; then
    [[ $EPAGES_DO_PATCH = 0 ]] && SKIP_PERL_UPDATE=skip_patch || SKIP_PERL_UPDATE=exit
  fi
  PKG_VERSION=$(pkg_installed -V epages-perl 2>/dev/null | tail -1)
  if [[ $SKIP_PERL_UPDATE = skip_patch ]] ; then
    # do nothing if we just unpack the package without starting the patch
    true
  elif [[ -n $SKIP_PERL_UPDATE ]] ; then
    # either show update message
    PKG_VERSION=${PKG_VERSION:+-$PKG_VERSION}
    echo "Package epages-perl${PKG_VERSION:+-$PKG_VERSION} is outdated."
    echo "First update epages-perl by executing:"
    echo "  ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl"
    echo "Then re-run: $0 $@"
    [[ $SKIP_PERL_UPDATE = exit ]] && exit 1
  else
    # or update epages-perl
    echo "ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl"
    ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl
  fi
fi

#------------------------------------------------------------------------------
# update perl if libssl.so is outdated and new is available
#------------------------------------------------------------------------------

# find latest lib32/ssl/libssl.so
LIBSSL=lib32/ssl/libssl.so

find_new_libssl () {
  local INSTALLED_VERSION=$(pkg_installed -V epages-perl 2>/dev/null)
  local NEW_LIBSSL=$(ls -v sh/60*[0-9]/EPAGES_PERL/$LIBSSL 2>/dev/null | tail -1)

  # return if no new libssl found
  [[ -n $NEW_LIBSSL ]] || return

  # if no package epages-perl found return new libssl
  [[ -n $INSTALLED_VERSION ]] || { echo "$NEW_LIBSSL" ; return; }

  local NEW_VERSION=${NEW_LIBSSL#sh/}
  NEW_VERSION=${NEW_VERSION%%/*}
  INSTALLED_VERSION=${INSTALLED_VERSION%%[~-]*}
  INSTALLED_VERSION=$(printf "%d%03d%03d%03d" ${INSTALLED_VERSION//./ })

  # if new version is greater than installed version return new libssl
  [[ $NEW_VERSION -gt $INSTALLED_VERSION ]] && echo "$NEW_LIBSSL" || echo up-to-date
}

NEW_LIBSSL=$(find_new_libssl)
echo "New libssl: $NEW_LIBSSL"

if [[ -f $NEW_LIBSSL ]] ; then
  # find version of latest lib32/ssl/libssl.so 
  SSL_VERSION=$(strings $NEW_LIBSSL | awk  '/^SSLv3 part of OpenSSL/{print $5}') 
  # if perl shall be updated and $EPAGES_PERL/lib32/ssl/libssl.so is not up-to-date
  if [[ -z $SKIP_PERL_UPDATE ]] && ! grep -q ${SSL_VERSION//\./\\\.} $EPAGES_PERL/$LIBSSL
  then
    for i in $(ls -d ${NEW_LIBSSL%/*}/*.so* 2>/dev/null) ; do
      # update library here
      echo "cat $i > $EPAGES_PERL/${i##*/EPAGES_PERL/}"
      cat $i > $EPAGES_PERL/${i##*/EPAGES_PERL/}
      PERL_HAS_CHANGED=1
    done
    # and remember this
    [[ -n $PERL_HAS_CHANGED ]] && touch ${LOG_DIR%/*}/PERL_HAS_CHANGED
  fi
fi

if [[ -n $NEW_LIBSSL ]] ; then
  # if perl shan't be updated or $EPAGES_PERL/lib32/ssl/libssl.so was updated
  if [[ -n $SKIP_PERL_UPDATE || -n $PERL_HAS_CHANGED || $NEW_LIBSSL = up-to-date ]]
  then
    for i in $(grep -l ${LIBSSL%/*}/ sh/*/patch.conf 2>/dev/null) ; do
      echo "Remove ${LIBSSL%/*}/ from: $i"
      # delete library entries in sh/*/patch.conf
      sed -i "s,^${LIBSSL%/*}/.*,," $i
      # sh/*/patch.conf has changed -> delete entry in filelist.txt
      [[ -f filelist.txt ]] || continue
      cp -p filelist.txt filelist.orig
      grep -v $i filelist.orig > filelist.txt
    done
  fi

  if [[ -f ${LOG_DIR%/*}/PERL_HAS_CHANGED && ! -f $LOG_DIR/PERL_WAS_RESTARTED ]] ; then
    # due to changed binaries we must restart the application server
    if [[ -x ${EPSERVICE:-"$EPAGES/Service/epagesService"} ]] ; then
      EPAGES_IGNORE=cron /etc/init.d/${EPAGES_PROJECT:-epages6} start_service
    fi
    touch $LOG_DIR/PERL_WAS_RESTARTED
  fi
fi

#------------------------------------------------------------------------------
# run perl
#------------------------------------------------------------------------------

date=$(date +'%Y-%h-%d-%H.%M.%S')

rm -f /tmp/epages-patch-result.log

$EPAGES_PERL/bin/perl patch.pl "$@" 2>&1 | tee $LOG_DIR/$date-${1////-}.log

# return success or failure

RESULT=${PIPESTATUS[0]}
if [[ -s /tmp/epages-patch-result.log ]] ; then
  PATCH_RESULT=$(tail -1 /tmp/epages-patch-result.log)
  [[ -n $PATCH_RESULT && -z ${PATCH_RESULT//[0-9]/} ]] && RESULT=$PATCH_RESULT
fi

exit $RESULT
